﻿using Microsoft.VisualBasic.FileIO;
using ProjectHotel.Entities;
using ProjectHotel.Entities.Enums;
using ProjectHotel.Repositories;
using ProjectHotel.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectHotel.Forms;

public partial class FormAthlete : Form
{
    private readonly IAthleteRepository _athleteRepository;
    private int? _athleteId;
    public int Id
    {
        set
        {
            try
            {
                var athlete = _athleteRepository.ReadAthleteById(value);
                if (athlete == null)
                {
                    throw new
                    InvalidDataException(nameof(athlete));
                }
                foreach (Sport elem in Enum.GetValues(typeof(Sport)))
                {
                    if ((elem & athlete.Sport) != 0)
                    {
                        checkedListBoxSport.SetItemChecked(checkedListBoxSport.Items.IndexOf(elem), true);
                    }
                }

                textBoxFirstName.Text = athlete.FirstName;
                textBoxLastName.Text = athlete.LastName;
                textBoxFatherName.Text = athlete.FatherName;
                comboBoxClass.SelectedItem = athlete.AthleteClass;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при получении данных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }
    }
    public FormAthlete(IAthleteRepository athleteRepository)
    {
        InitializeComponent();
        _athleteRepository = athleteRepository ??
            throw new ArgumentNullException(nameof(athleteRepository));
        foreach (var elem in Enum.GetValues(typeof(Sport)))
        {
            checkedListBoxSport.Items.Add(elem);
        }
        comboBoxClass.DataSource = Enum.GetValues(typeof(AthleteClass));
    }
    private void buttonSave_Click(object sender, EventArgs e)
    {
        try
        {
            if (string.IsNullOrWhiteSpace(textBoxFirstName.Text) ||
                string.IsNullOrWhiteSpace(textBoxLastName.Text) ||
                string.IsNullOrWhiteSpace(textBoxFatherName.Text) ||
                comboBoxClass.SelectedIndex < 0 || checkedListBoxSport.SelectedIndex < 0)
            {
                throw new Exception("Имеются незаполненные поля");
            }
            if (_athleteId.HasValue)
            {
                _athleteRepository.UpdateAthlete(CreateAthlete(_athleteId.Value));
            }
            else
            {
                _athleteRepository.CreateAthlete(CreateAthlete(0));
            }
            Close();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка при сохранении",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }
    private void buttonCancel_Click(object sender, EventArgs e) =>
    Close();
    private Athlete CreateAthlete(int id)
    {
        Sport sport = Sport.None;
        foreach (var elem in checkedListBoxSport.CheckedItems)
        {
            sport |= (Sport)elem;
        }
        return Athlete.CreateEntity(id, textBoxFirstName.Text, textBoxLastName.Text,
            textBoxFatherName.Text, sport, (AthleteClass)comboBoxClass.SelectedItem!);
    }

}
