﻿using ProjectHotel.Entities;
using ProjectHotel.Repositories;
using ProjectHotel.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectHotel.Forms;

public partial class FormHotel : Form
{
    private readonly IHotelRepository _hotelRepository;
    private int? _hotelId;
    public int Id
    {
        set
        {
            try
            {
                var hotel =
                _hotelRepository.ReadHotelById(value);
                if (hotel == null)
                {
                    throw new
    InvalidDataException(nameof(hotel));
                }
                textBoxHotelName.Text = hotel.Name;
                textBoxHotelAddress.Text =
                hotel.Address;

                _hotelId = value;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при получении данных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }
    }
    public FormHotel(IHotelRepository hotelRepository)
    {
        InitializeComponent();
        _hotelRepository = hotelRepository ??
        throw new
        ArgumentNullException(nameof(hotelRepository));
    }
    private void ButtonSave_Click(object sender, EventArgs e)
    {
        try
        {
            if (string.IsNullOrWhiteSpace(textBoxHotelName.Text)
            ||
            string.IsNullOrWhiteSpace(textBoxHotelAddress.Text))
            {
                throw new Exception("Имеются незаполненные поля");
            }
            if (_hotelId.HasValue)
            {
                _hotelRepository.UpdateHotel(CreateHotel(_hotelId.Value));
            }
            else
            {
                _hotelRepository.CreateHotel(CreateHotel(0));
            }
            Close();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка при сохранении",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }
    private void ButtonCancel_Click(object sender, EventArgs e) => Close();
    private Hotel CreateHotel(int id) => Hotel.CreateEntity(id, textBoxHotelName.Text, textBoxHotelAddress.Text);

    
}

