﻿using Microsoft.Extensions.Logging;
using ProjectHotel.Repositories;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectHotel.Reports;

internal class DocReport
{
    private readonly IAthleteRepository _athleteRepository;
    private readonly IRoomRepository _roomRepository;
    private readonly IHotelRepository _hotelRepository;
    private readonly ILogger<DocReport> _logger;
    public DocReport(IAthleteRepository athleteRepository, IRoomRepository roomRepository, IHotelRepository hotelRepository, ILogger<DocReport> logger)
    {
        _athleteRepository = athleteRepository ?? throw new ArgumentNullException(nameof(athleteRepository));
        _roomRepository = roomRepository ?? throw new ArgumentNullException(nameof(roomRepository));
        _hotelRepository = hotelRepository ?? throw new ArgumentNullException(nameof(hotelRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }
    public bool CreateDoc(string filePath, bool includeAthletes, bool includeRooms, bool includeHotels)
    {
        try
        {
            var builder = new WordBuilder(filePath)
            .AddHeader("Документ со справочниками");
            if (includeAthletes)
            {
                builder.AddParagraph("Спортсмены")
                .AddTable([2400, 2400, 2400, 1200, 1200],
                GetAthletes());
            }
            if (includeRooms)
            {
                builder.AddParagraph("Комнаты")
                .AddTable([2400, 2400, 2400], GetRooms());
            }
            if (includeHotels)
            {
                builder.AddParagraph("Гостиницы")
                .AddTable([2400, 2400], GetHotels());
            }
            builder.Build();
            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }
    private List<string[]> GetAthletes()
    {
        return [
        ["Имя", "Фамилия", "Отчество", "Вид/Виды спорта", "Разряд"],
            .. _athleteRepository.ReadAthletes().Select(x => new string[] { x.FirstName,
x.LastName,x.FatherName, x.Sport.ToString(), x.AthleteClass.ToString() }),
        ];
    }
    private List<string[]> GetRooms()
    {
        return [
        ["Отель", "Номер", "Вместимость"],
            .. _roomRepository
        .ReadRooms()
        .Select(x => new string[] { x.HotelId.ToString(), x.Name,
x.Capacity.ToString() }),
        ];
    }
    private List<string[]> GetHotels()
    {
        return [
        ["Название", "Адрес"],
            .. _hotelRepository
        .ReadHotels()
        .Select(x => new string[] { x.Name,
x.Address}),
        ];
    }
}
