﻿using Dapper;
using Microsoft.Extensions.Logging;
using Npgsql;
using ProjectHotel.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Newtonsoft.Json;


namespace ProjectHotel.Repositories.Implementations;

public class AthleteRepository : IAthleteRepository
{
    private readonly IConnectionString _connectionString;

    private readonly ILogger<AthleteRepository> _logger;

    public AthleteRepository(IConnectionString connectionString, ILogger<AthleteRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }

    public void CreateAthlete(Athlete athlete)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(athlete));

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            var queryInsert = @"
                INSERT INTO Athletes (FirstName, LastName, FatherName, Sport, AthleteClass) 
                VALUES (@FirstName, @LastName, @FatherName, @Sport, @AthleteClass)";
            connection.Execute(queryInsert, athlete);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }
    public void DeleteAthlete(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            var queryDelete = @"
                DELETE FROM Athletes
                WHERE Id = @id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }
    public Athlete ReadAthleteById(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
                SELECT * FROM Athletes WHERE Id = @Id;
            ";
            var athlete = connection.QueryFirst<Athlete>(querySelect, new { id });
            _logger.LogDebug("Найденный объект: {json}", JsonConvert.SerializeObject(athlete));
            return athlete;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }
    public IEnumerable<Athlete> ReadAthletes()
    {
        _logger.LogInformation("Получение всех объектов");

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            var querySelect = @"SELECT * FROM Athletes";
            var athletes = connection.Query<Athlete>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(athletes));
            return athletes;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при получении всех объектов");
            throw;
        }
    }
    public void UpdateAthlete(Athlete athlete)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(athlete));

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"
                UPDATE Athletes
                SET FirstName = @FirstName, LastName = @LastName, FatherName = @FatherName, @Sport = Sport, @AthleteClass = AthleteClass
                WHERE Id = @Id;
            ";
            connection.Execute(queryUpdate, athlete);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }
}
