﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectHotel.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectHotel.Repositories.Implementations;

internal class HotelRepository : IHotelRepository
{
    private readonly IConnectionString _connectionString;

    private readonly ILogger<HotelRepository> _logger;
    public HotelRepository(IConnectionString connectionString, ILogger<HotelRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }

    public IEnumerable<Hotel> ReadHotels()
    {
        _logger.LogInformation("Получение всех объектов");

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            var querySelect = @"SELECT * FROM Hotels";
            var hotel = connection.Query<Hotel>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(hotel));
            return hotel;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при получении всех объектов");
            throw;
        }
    }
    public Hotel ReadHotelById(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
                SELECT * FROM Hotels WHERE Id = @Id;
            ";
            var hotel = connection.QueryFirst<Hotel>(querySelect, new { id });
            _logger.LogDebug("Найденный объект: {json}", JsonConvert.SerializeObject(hotel));
            return hotel;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }
    public void CreateHotel(Hotel hotel)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(hotel));

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            var queryInsert = @"
                INSERT INTO Hotels (Name, Address) 
                VALUES (@Name, @Address)";
            connection.Execute(queryInsert, hotel);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }
    public void UpdateHotel(Hotel hotel)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(hotel));

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"
                UPDATE Hotels
                SET Name = @Name, Address = @Address
                WHERE Id = @Id;
            ";
            connection.Execute(queryUpdate, hotel);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }

    public void DeleteHotel(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            var queryDelete = @"
                DELETE FROM Hotels
                WHERE Id = @id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }
}
