﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectHotel.Entities;
using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectHotel.Repositories.Implementations;

internal class PlacingAthleteRepository : IPlacingAthleteRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<PlacingAthleteRepository> _logger;

    public PlacingAthleteRepository(IConnectionString connectionString, ILogger<PlacingAthleteRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }

    public void CreatePlacingAthlete(PlacingAthlete placingAthlete)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(placingAthlete));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            using var transaction = connection.BeginTransaction();

            var queryInsert = @"
                INSERT INTO PlacingAthlete (RoomId, PlacingDate) 
                VALUES (@RoomId, @PlacingDate);
                SELECT MAX(Id) FROM PlacingAthlete;
            ";
            var placingAthleteId = connection.QueryFirst<int>(queryInsert, placingAthlete, transaction);

            var querySubInsert = @"
                INSERT INTO AthletePlacingAthlete (AthleteId, PlacingAthleteId, LengthOfStay)
                VALUES (@AthleteId, @PlacingAthleteId, @LengthOfStay);
            ";
            foreach (var elem in placingAthlete.AthletePlacingAthletes)
            {
                connection.Execute(querySubInsert, new { elem.AthleteId, placingAthleteId , elem.LengthOfStay }, transaction);
            }

            transaction.Commit();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }


    public void DeletePlacingAthlete(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
                DELETE FROM PlacingAthlete
                WHERE Id = @id
            ";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public IEnumerable<PlacingAthlete> ReadPlacingAthlete(int? roomId = null, DateTime? placing = null)
    {
        _logger.LogInformation("Получение всех объектов");

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"SELECT * FROM PlacingAthlete";
            var placingAthletes = connection.Query<PlacingAthlete>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(placingAthletes));
            return placingAthletes;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }
}
