﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectHotel.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectHotel.Repositories.Implementations
{
    internal class RoomRepository : IRoomRepository
    {
        private readonly IConnectionString _connectionString;

        private readonly ILogger<RoomRepository> _logger;
        public RoomRepository(IConnectionString connectionString, ILogger<RoomRepository> logger)
        {
            _connectionString = connectionString;
            _logger = logger;
        }
        public IEnumerable<Room> ReadRooms()
        {
            _logger.LogInformation("Получение всех объектов");

            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var querySelect = @"SELECT * FROM Rooms";
                var rooms = connection.Query<Room>(querySelect);
                _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(rooms));
                return rooms;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при получении всех объектов");
                throw;
            }
        }
        public Room ReadRoomById(int id)
        {
            _logger.LogInformation("Получение объекта по идентификатору");
            _logger.LogDebug("Объект: {id}", id);

            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = @"
                SELECT * FROM Rooms WHERE Id = @Id;
            ";
                var room = connection.QueryFirst<Room>(querySelect, new { id });
                _logger.LogDebug("Найденный объект: {json}", JsonConvert.SerializeObject(room));
                return room;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при поиске объекта");
                throw;
            }
        }
        public void CreateRoom(Room rooms)
        {
            _logger.LogInformation("Добавление объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(rooms));

            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryInsert = @"
                INSERT INTO Rooms (HotelId, Name,Capacity) 
                VALUES (@HotelId, @Name, @Capacity)";
                connection.Execute(queryInsert, rooms);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }
        public void UpdateRoom(Room rooms)
        {

        }
        public void DeleteRoom(int id)
        {
            _logger.LogInformation("Удаление объекта");
            _logger.LogDebug("Объект: {id}", id);

            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryDelete = @"
                DELETE FROM Rooms
                WHERE Id = @id";
                connection.Execute(queryDelete, new { id });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при удалении объекта");
                throw;
            }
        }

    }
}
